package demo.client;

import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.History;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.MenuBar;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

import demo.client.dtos.UserPassKeyDto;
import demo.client.login.LoginFormPresenter;
import demo.client.login.LoginFormView;

public class Environment {
	
	
	protected class HistoryCommand implements Command {
		String historyToken;

		public HistoryCommand(final String newToken) {
			historyToken = newToken;
		}

		public void execute() {
			launch(historyToken);
		}
	}

	final Model model;
	final Grid rootDisplay = new Grid(2, 1);
	final MenuBar runMenuBar = new MenuBar();
	final VerticalPanel runPanel = new VerticalPanel();
	String startingToken;

	String currentUser;
	String currentPassword;

	Command sorry = new Command() {
		@Override
		public void execute() {
			showAlert("Sorry, this isn't ready yet.");
		}
	};

	public Environment(final Model aModel, final String aToken) {
		model = aModel;
		startingToken = aToken;
	}

	public String getCurrentUserName() {
		return currentUser;
	}

	public String getCurrentUserPassword() {
		return currentPassword;
	}

	public Model getModel() {
		return model;
	}

	public void launch(final String token) {
		launch(token, null);
	}

	public void launch(String token, Panel panel) {

		/*
		 * If no panel is given, use the standard runPanel, and add the token to
		 * History.
		 * 
		 * Be careful not to forget the "false" parameter in newItem(...), or
		 * programs will be called twice!
		 */
		if (panel == null) {
			panel = runPanel;
			History.newItem(token, false);
		}
		panel.clear();

		// TODO Check, before running, if the current user is
		// allowed to run the program

		if (token.isEmpty()) {
			// no need to do anything...
		} else if (token.equals(LoginFormPresenter.PLACE)) {
			showLogin(RootPanel.get());
		} else {
			showAlert("Unrecognized token=" + token);
		}
	}

	public void setCurrentUserName(final String s) {
		currentUser = s;
	}

	public void setCurrentUserPassword(final String s) {
		currentPassword = s;
	}

	public void showAlert(final String alertText) {
		Window.alert(alertText);
	}

	private void showLogin(final Panel panel) {
		setCurrentUserName("");

		final LoginFormPresenter loginForm = new LoginFormPresenter("", new LoginFormView(), this, new SimpleCallback<UserPassKeyDto>() {
			@Override
			public void goBack(final UserPassKeyDto result) {
				setCurrentUserName(result.user);
				setCurrentUserPassword(result.pass);
				showMainMenu();
			}
		});

		panel.clear();
		panel.add(loginForm.getDisplay().asWidget());
	}

	private void showMainMenu() {
		// TODO Use user information for menu configuration

		runMenuBar.clearItems();
		runMenuBar.setWidth("100%");
		createMenu(runMenuBar);

		rootDisplay.setWidth("100%");
		rootDisplay.setWidget(0, 0, runMenuBar);
		rootDisplay.setWidget(1, 0, runPanel);

		RootPanel.get().clear();
		RootPanel.get().add(rootDisplay);

		/*
		 * If the application was started with a token, now that the user is
		 * logged in, it's time to show it. Don't forget to clear startingToken,
		 * or after a logout/login, we will go back again to it.
		 */
		if (!startingToken.isEmpty()) {
			launch(startingToken);
			startingToken = "";
		}
	}

	private void createMenu(MenuBar mb) {
		final MenuBar mbLogin = new MenuBar(true);
		mbLogin.addItem("Login again", new HistoryCommand(LoginFormPresenter.PLACE));
		mb.addItem("Login", mbLogin);
		final MenuBar mbDummy = new MenuBar(true);
		mbDummy.addItem("dummy 1", new HistoryCommand("dummy"));
		mb.addItem("Dummy", mbLogin);
	}
}
